@@ -10,21 +10,23 @@ except ImportError:
     raise ImportError('Using an encrypted field requires the Keyczar module. '
                       'You can obtain Keyczar from http://www.keyczar.org/.')
 
-class EncryptionWarning(RuntimeWarning): pass
+
+class EncryptionWarning(RuntimeWarning):
+    pass
+
 
 class BaseEncryptedField(models.Field):
     prefix = 'enc_str:::'
+
     def __init__(self, *args, **kwargs):
         if not hasattr(settings, 'ENCRYPTED_FIELD_KEYS_DIR'):
-            raise ImproperlyConfigured('You must set the '
-                'ENCRYPTED_FIELD_KEYS_DIR setting to your Keyczar keys directory.')
+            raise ImproperlyConfigured('You must set the ENCRYPTED_FIELD_KEYS_DIR setting to your Keyczar keys directory.')
         self.crypt = keyczar.Crypter.Read(settings.ENCRYPTED_FIELD_KEYS_DIR)
 
         # Encrypted size is larger than unencrypted
         self.unencrypted_length = max_length = kwargs.get('max_length', None)
         if max_length:
-            max_length = len(self.prefix) + \
-                len(self.crypt.Encrypt('x'*max_length))
+            max_length = len(self.prefix) + len(self.crypt.Encrypt('x' * max_length))
             # TODO: Re-examine if this logic will actually make a large-enough
             # max-length for unicode strings that have non-ascii characters in them.
             kwargs['max_length'] = max_length
@@ -32,7 +34,9 @@ class BaseEncryptedField(models.Field):
         super(BaseEncryptedField, self).__init__(*args, **kwargs)
 
     def to_python(self, value):
-        if value and (value.startswith(self.prefix)):
+        if isinstance(self.crypt.primary_key, keyczar.keys.RsaPublicKey):
+            retval = value
+        elif value and (value.startswith(self.prefix)):
             retval = self.crypt.Decrypt(value[len(self.prefix):])
             if retval:
                 retval = retval.decode('utf-8')
@@ -50,8 +54,9 @@ class BaseEncryptedField(models.Field):
             # so truncate before encryption
             max_length = self.unencrypted_length
             if max_length and len(value) > max_length:
-                warnings.warn("Truncating field %s from %d to %d bytes" % \
-                    (self.name, len(value), max_length),EncryptionWarning)
+                warnings.warn("Truncating field %s from %d to %d bytes" % (
+                    self.name, len(value), max_length), EncryptionWarning
+                )
                 value = value[:max_length]
 
             value = self.prefix + self.crypt.Encrypt(value)
@@ -82,8 +87,8 @@ class EncryptedTextField(BaseEncryptedField):
 class EncryptedCharField(BaseEncryptedField):
     __metaclass__ = models.SubfieldBase
 
-    def __init__(self, max_length=None, *args, **kwargs):
-        super(EncryptedCharField, self).__init__(max_length=max_length, *args, **kwargs)
+    def __init__(self, *args, **kwargs):
+        super(EncryptedCharField, self).__init__(*args, **kwargs)
 
     def get_internal_type(self):
         return "CharField"
